#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 3/1/21

# This plugin was reproduced in GIMP Python with new adbons from the original
# wrap_effect.scm iccii Elsamuko.

# New colorazation and Smooth option non-interactive with Spnner.

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.#

# 
# --------------------------------------------------------------------
#   ## Original information ## 
#   Wrap paint effect Script  for GIMP 1.2
#   Created after following the script:
#   wrap_effect
#	Draws with wrap effect
#   Draws with wrap effect, which simulates Photoshop's Wrap filter
#	Iccii <iccii@hotmail.com
#	Iccii
#	Oct, 2001


from gimpfu import *
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def wrap_effect_with_color_opt(inImg, inLayer, colorize, color, inRadius, inGamma1, inGamma2, inSmooth, smoothness):
          
    #Initiates the temporary state
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()		
    
    pdb.gimp_image_undo_group_start(inImg)
    
    #Make active the image and save to channel 
    inLayer = pdb.gimp_image_get_active_drawable(inImg)       
    channel = pdb.gimp_selection_save(inImg)   
    pdb.gimp_image_set_active_layer(inImg, inLayer)
    
    # Create an image duplicate
    non_empty = pdb.gimp_edit_copy(inLayer)
    newlayer = pdb.gimp_layer_new(inImg, inImg.width, inImg.height, RGBA_IMAGE, "New layer", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(inImg, newlayer, None, 0)
    pdb.gimp_layer_add_alpha(newlayer)
    newlayer = pdb.gimp_edit_paste(newlayer, FALSE)
    pdb.gimp_floating_sel_anchor(newlayer) 
    newlayer = pdb.gimp_image_get_layer_by_name(inImg, "New layer")
    pdb.gimp_item_set_name(newlayer, "Wrap effect")
    
    #Desaturate the image
    pdb.gimp_drawable_desaturate(newlayer, DESATURATE_LIGHTNESS)
    
    #Blur the image
    pdb.plug_in_gauss_iir2(inImg, newlayer, inRadius, inRadius)
    
    #Apply edge filter
    pdb.plug_in_edge(inImg, newlayer, 2.0, 1, 4)
    
    #Invert the image
    pdb.gimp_invert(newlayer)
        
    #For extra smooth  
    if inSmooth == True:
        pdb.plug_in_gauss_iir2(inImg, newlayer, smoothness, smoothness)
                
    #Create a layer mask
    newMask = pdb.gimp_layer_create_mask(newlayer, ADD_MASK_BLACK)
    pdb.gimp_layer_add_mask(newlayer, newMask)
   
    #Copy the image and paste it on the layer mask
    non_empty = pdb.gimp_edit_copy(newlayer)
    newMaskk = pdb.gimp_edit_paste(newMask, FALSE)
    pdb.gimp_floating_sel_anchor(newMaskk) 
     
    #Apply levels to the image and the layer mask   
    pdb.gimp_levels(newlayer, HISTOGRAM_VALUE, 0, 255, inGamma1/10, 0, 255)
    pdb.gimp_levels(newlayer, HISTOGRAM_VALUE, 0, 255, inGamma1/10, 0, 255)
    pdb.gimp_levels(newMask, HISTOGRAM_VALUE, 0, 255, inGamma2/10, 0, 255)
    pdb.gimp_levels(newMask, HISTOGRAM_VALUE, 0, 255, inGamma2/10, 0, 255)
    
    #Apply layer mask and merge the layer
    pdb.gimp_layer_remove_mask(newlayer, MASK_APPLY)
    newlayer = pdb.gimp_image_merge_down(inImg, newlayer, CLIP_TO_IMAGE)
    
    if colorize == True:
        newlayer2 = pdb.gimp_layer_new(inImg, inImg.width, inImg.height, RGB, "Color overlay", 100, LAYER_MODE_OVERLAY)
        pdb.gimp_image_add_layer(inImg, newlayer2, 0)
        pdb.gimp_layer_add_alpha(newlayer2)
        pdb.gimp_drawable_fill(newlayer2, FILL_TRANSPARENT)
        pdb.gimp_context_set_foreground(color)
        pdb.gimp_edit_fill(newlayer2, FILL_FOREGROUND)
        newlayer = pdb.gimp_image_merge_down(inImg, newlayer2, CLIP_TO_IMAGE)
    
    #Remove the channel and set active the layer
    pdb.gimp_image_remove_channel(inImg, channel)  
    pdb.gimp_image_set_active_layer(inImg, newlayer)
        
	#Set gimp to default
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
    pdb.gimp_displays_flush()
    pdb.gimp_image_undo_group_end(inImg)

register(
    "wrap_effect_with_color_opt",
    "Draws with wrap effect with Color option",
    "Draws with wrap effect which simulates Photoshop's Wrap filter",
    "Pocholo M",
    "Pocholo M",
    "2021",
    "Wrap Effect",
    "RGB*,GRAY*",
    [    
    (PF_IMAGE, "inImg", "Input image", None),
    (PF_LAYER, "inLayer", "Input layer", None),
    (PF_TOGGLE, "colorize", "Colorize?", FALSE),
    (PF_COLOR, "color", "Color", (230, 34, 34)),
    (PF_SPINNER,	"inRadius", "Randomness", 10.0, (0, 32.0, 1)),
    (PF_SPINNER,	"inGamma1", "Highlight Balance", 2.0, (1.0, 10, 0.5)),
    (PF_SPINNER,	"inEdge", "Edge Amount", 2.0, (1.0, 10, 0.5)),
    (PF_TOGGLE,	"inSmooth", "Smooth?", FALSE),
    (PF_SPINNER,  "smoothness", "Smoothness", 1.5, (1.0, 20, 0.5)),   
    ],
    [],
    wrap_effect_with_color_opt, menu="<Image>/Filters/Distorts/Wrap Effect",
    domain=("gimp20", gimp.locale_directory))
main()				